@@ -22,6 +22,30 @@
 //!     sop::cli::main(&MySOPImplementation::default());
 //! }
 //! ```
+//!
+//! ## Generating shell completions
+//!
+//! To create shell completions, add this snippet to `Cargo.toml`:
+//!
+//! ```toml
+//! [build-dependencies]
+//! sop = "..."
+//! ```
+//!
+//! And create `build.rs` along the lines of:
+//!
+//! ```rust,no_run
+//! #[cfg(feature = "cli")]
+//! fn main() {
+//!     let outdir = std::env::var_os("CARGO_TARGET_DIR")
+//!         .or(std::env::var_os("OUT_DIR"))
+//!         .expect("cargo to set OUT_DIR");
+//!     sop::cli::write_shell_completions("sqop", outdir).unwrap();
+//! }
+//!
+//! #[cfg(not(feature = "cli"))]
+//! fn main() {}
+//! //! ```
 
 use std::{
     io,
@@ -153,6 +177,27 @@ enum Operation {
     Unsupported(Vec<String>),
 }
 
+/// Generates shell completions.
+pub fn write_shell_completions<B, P>(binary_name: B,
+                                     out_path: P)
+                                     -> io::Result<()>
+where B: AsRef<str>,
+      P: AsRef<Path>,
+{
+    use structopt::clap::Shell;
+
+    let binary_name = binary_name.as_ref();
+    let out_path = out_path.as_ref();
+    std::fs::create_dir_all(&out_path)?;
+
+    let mut clap = Operation::clap();
+    for shell in &[Shell::Bash, Shell::Fish, Shell::Zsh, Shell::PowerShell,
+                   Shell::Elvish] {
+        clap.gen_completions(binary_name, *shell, out_path);
+    }
+    Ok(())
+}
+
 /// Implements the SOP command-line interface.
 pub fn main(sop: &dyn crate::SOP) -> ! {
     use std::process::exit;
